/**************************************************************************************************************************************************
*       USI_I2C.c Set's up the MSP430 uC USI in I2C mode.                                                                                         *
*                                                                                                                                                 *
*       Initiating calls:   main                                                - Called from IAR's reset                                         *
*                                                                                                                                                 *
*       Author:             Mike Claassen                                                                                                         *
*                                                                                                                                                 *
*       Revision Date:      February 2011                                                                                                         *
*                                                                                                                                                 *
*       Revision Level:     1                                                                                                            *
*                                                                                                                                                 *
*                                               *
*                                                                                                                                                 *
*
 * Copyright (C) 2017 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*

***************************************************************************************************************************************************
*                                 MODULE CHANGE LOG                                                                                               *
*                                                                                                                                                 *
*       Date Changed:             {date of change}                Developer:       {developer name}                                               *
*       Change Description:       {describe change}                                                                                               *
*                                                                                                                                                 *
**************************************************************************************************************************************************/
/**************************************************************************************************************************************************
*                                 Included Headers                                                                                                *
**************************************************************************************************************************************************/
#include <msp430.h>                                                        // Definition Files for the MSP430
/**************************************************************************************************************************************************
*                                 Definitions                                                                                                     *
**************************************************************************************************************************************************/
/************************** I2C SCI Engine ************************************/
#define I2C_SDA 0x80                                                                 // SDA on P1.7
#define I2C_SCL 0x40                                                                 // SCL on P1.6


//   General I2C State Machine
#define I2C_START_CONDITION               0x00
#define I2C_READ_ADDRESS_ACK_NAK          0x01
#define I2C_CHECK_ACK_NAK_ADDRESS         0x02
#define I2C_READ_ACK_NAK_REGISTER         0x03
#define I2C_CHECK_ACK_NAK_REGISTER        0x04
#define I2C_PROCESS_STOP_CONDITION        0x05
#define I2C_STOP_CONDITION                0x06

//  Read State Machine
#define I2C_RESTART_CONDITION             0x10
#define I2C_READ_ACK_NAK_RESTART          0x12
#define I2C_CHECK_ACK_NAK_RESTART         0x14
#define I2C_SEND_ACK_NAK_DATA_READ        0x16
#define I2C_WORD_DATA_READ                0x18

// Write State Machine
#define I2C_READ_ACK_NAK_DATA_WRITE       0x20
#define I2C_CHECK_ACK_NAK_DATA_WRITE      0x22
#define I2C_READ_ACK_NAK_DATA_LSB_WRITE   0x24

#define I2C_WRITE 0x00
#define I2C_READ 0x01

#define I2C_BYTE 0x00
#define I2C_WORD 0x01

#define I2C_ACK  0x00
#define I2C_NAK  0x01
                                                     
/**************************************************************************************************************************************************
*                                 Prototypes                                                                                                      *
**************************************************************************************************************************************************/
/************ System Configuration *************************/
//void Set_I2C(void);

/************ I2C Interface *******************************/
unsigned int I2C_Read (unsigned char, unsigned char, unsigned char);
void I2C_Write (unsigned char, unsigned char, unsigned char, unsigned int);
void clear_I2C_State_Machine (void);
void read_ACK_NAK_bit (void);
void transmit_ACK_NAK_bit (unsigned char);
void initiate_I2C_stop (void);
void send_I2C_byte (unsigned char);
void read_I2C_byte (void);

/**************************************************************************************************************************************************
*                                 External Variables                                                                                              *
**************************************************************************************************************************************************/

/**************************************************************************************************************************************************
*                                 Global Variables                                                                                                *
**************************************************************************************************************************************************/

/************ I2C Interface *******************************/
unsigned char Read_Write_Mode = I2C_WRITE;                                      // I2C mode used to decide if Repeat Start is needed
unsigned char Byte_Word_Mode = I2C_WORD;                                        // I2C mode used to decide 1 or 2 byte read/write
unsigned int  MST_Data = 0x00;                                                  // Variable for transmitted data
unsigned char SLV_Addr = 0x00;                                                  // I2C Address of Device
unsigned char REG_Addr = 0x00;                                                  // Address in the device being set/read
unsigned int  I2C_State = 0x00;                                                 // State variable
unsigned char I2C_State_Table [14];                                             // Used to troublehoot the I2C state machine */

/**************************************************************************************************************************************************
*                                 I2C  - Taken from MSP430x20x3_usi_07.c                                                                          *
**************************************************************************************************************************************************/
void Set_I2C(void)
{
	__disable_interrupt(); // FROM TEST CODE
    USICTL0 = USIPE6+USIPE7+USIMST+USISWRST;                                    // Port & USI mode setup
    USICTL1 = USII2C+USIIE;                                                     // Enable I2C mode & USI interrupt
    USICKCTL = USIDIV_4+USISSEL_2+USICKPL;                                      // Setup USI clocks: SCL = SMCLK/16 (~62.5kHz)
    USICNT |= USIIFGCC;                                                         // Disable automatic clear control
    USICTL0 &= ~USISWRST;                                                       // Enable USI
    USICTL1 &= ~USIIFG;                                                         // Clear pending flag
    __enable_interrupt(); // FROM TEST CODE
}

unsigned int I2C_Read (unsigned char Word_Byte_mode, unsigned char Slave_Address, unsigned char Register_Address)
{
    Read_Write_Mode = I2C_READ;                                                 // I2C mode used to decide if Repeat Start is needed
    Byte_Word_Mode = Word_Byte_mode;                                            // I2C mode used to decide 1 or 2 byte read/write
    SLV_Addr = Slave_Address<<1;                                                // I2C Address of Device Shifted by 1 for Read/Write bit
    REG_Addr = Register_Address;                                                // Address in the device being set/read
    MST_Data = 0x00;                                                            // Clear the Data will be updated by the I2C Read
    USICTL1 |= USIIFG;                                                          // Set flag and start communication

    while (I2C_State != I2C_START_CONDITION);                                   // Wait for START Condition State
    return MST_Data;
}

void I2C_Write (unsigned char Word_Byte_mode, unsigned char Slave_Address, unsigned char Register_Address, unsigned int Register_Data)
{
    Read_Write_Mode = I2C_WRITE;                                                // I2C mode used to decide if Repeat Start is needed
    Byte_Word_Mode = Word_Byte_mode;                                            // I2C mode used to decide 1 or 2 byte read/write
    SLV_Addr = Slave_Address<<1;                                                // I2C Address of Device Shifted by 1 for Read/Write bit
    REG_Addr = Register_Address;                                                // Address in the device being set/read
    MST_Data = Register_Data;                                                   // Variable for transmitted data
    USICTL1 |= USIIFG;                                                          // Set flag and start communication

    while (I2C_State != I2C_START_CONDITION);                                   // Wait for START Condition State
}

void send_I2C_byte (unsigned char I2C_data)
{
    USICTL0 &= ~USIGE;                                                          // Set the Latch to Gated by Clock
    USICTL0 |= USIOE;                                                           // Set SDA = output
    USISRL = I2C_data;                                                          // Load I2C byte in the device to be read/written
    USICNT |=  0x08;                                                            // Bit counter = 8, start TX
}

void read_I2C_byte (void)
{
    USICTL0 &= ~USIOE;                                                          // Set SDA = input
    USICNT |=  0x08;                                                            // Bit counter = 8, start RX
}

void read_ACK_NAK_bit (void)
{
    USICTL0 &= ~(USIOE + USIGE);                                                // Set SDA = input and Latch to be Gated
    USICNT |= 0x01;                                                             // Bit counter = 1, receive (N)Ack bit
}

void transmit_ACK_NAK_bit (unsigned char ACK_NAK)
{
    USICTL0 |= USIOE;                                                           // SDA = output
    USISRL = ACK_NAK;                                                           // Set shift register low, Sending an Ack or high sending an Nak
    USICNT |= 0x01;                                                             // Bit counter = 1, send (N)Ack bit
}

void initiate_I2C_stop (void)
{
    USICTL0 |= USIOE;                                                           // Set SDA = output
    USICTL0 &= ~USIGE;                                                          // Set the Latch to Gated by Clock
    USISRL = 0x00;                                                              // Set shift register LSB to 0x00 making SDA low
    USICNT |= 0x01;                                                             // Clock 1 Bit by setting Bit counter making SCL high 
}

void clear_I2C_State_Machine (void)
{
    unsigned char i = sizeof(I2C_State_Table);  
    unsigned char y;
  
    for (y=0; y<=i; y++)
    {
        I2C_State_Table [y] = 0x00;
    }
}

/**************************************************************************************************************************************************
*    USI interrupt service routine used for I2C State Macine                                                                                      *
**************************************************************************************************************************************************/
/*
Transmit Register in Device

    Start       7 bit Address         W   ACK         Device Address                Register Address
      _    _______________________    _   _    ___________________________     ___________________________
_   _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ |
SCL  |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|

Followed by either Transmit of Data to the Register:

           Device High Byte                Device Low Byte
       ___________________________     ___________________________
_   _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _
SCL  |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|

or Receive Data from the Register:

 Re-Start       7 bit Address         R   ACK      Device High Byte                Device Low Byte
      _    _______________________    _   _    ___________________________     ___________________________
_   _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ | _ |
SCL  |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|

//   General I2C State Machine
#define I2C_START_CONDITION               0x00
#define I2C_READ_ADDRESS_ACK_NAK          0x01
#define I2C_CHECK_ACK_NAK_ADDRESS         0x02
#define I2C_READ_ACK_NAK_REGISTER         0x03
#define I2C_CHECK_ACK_NAK_REGISTER        0x04
#define I2C_PROCESS_STOP_CONDITION        0x05
#define I2C_STOP_CONDITION                0x06

//  Read State Machine
#define I2C_RESTART_CONDITION             0x10
#define I2C_READ_ACK_NAK_RESTART          0x12
#define I2C_CHECK_ACK_NAK_RESTART         0x14
#define I2C_SEND_ACK_NAK_DATA_READ        0x16
define  I2C_WORD_DATA_READ                0x18

// Write State Machine
#define I2C_READ_ACK_NAK_DATA_WRITE       0x20
#define I2C_CHECK_ACK_NAK_DATA_WRITE      0x22
#define I2C_READ_ACK_NAK_DATA_LSB_WRITE   0x24


                                   Receive Path (WORD)
                     Start Condition + 8 bit Address plus Write                                            (I2C_START_CONDITION)             - 0x00
                                 Read ACk bit (1 Bit)                                                      (I2C_READ_ADDRESS_ACK_NAK)        - 0x01
    _____________________________________|______________________________________
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_ADDRESS)       - 0x02
Send Stop Condition                                             Transmit Register Address (8 bits)         (I2C_CHECK_ACK_NAK_ADDRESS)
Clear I2C Bus (I2C_STOP_CONDITION)                                    Read ACK Bit (1Bit)                  (I2C_READ_ACK_NAK_REGISTER)       - 0x03
    ___________________________________________________________________________|
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_REGISTER)      - 0x04
Send Stop Condition                                                            |
Clear I2C Bus (I2C_STOP_CONDITION)                                             |
                                          _____________________________________|
                                         |
                                 (Re)Start Condition                                                       
                                8 bit Address plus Read                                                    (I2C_RESTART_CONDITION)          - 0x10
                                         |
                                 Read ACk bit (1 Bit)                                                      (I2C_READ_ACK_NAK_RESTART)       - 0x12
    _____________________________________|______________________________________
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_RESTART)      
Send Stop Condition                                             Read Register Data LSB (8 bits)            (I2C_CHECK_ACK_NAK_RESTART)      - 0x14
Clear I2C Bus (I2C_STOP_CONDITION)                                    Send ACK Bit (1Bit)                  (I2C_SEND_ACK_NAK_DATA_READ)     - 0x16
    ___________________________________________________________________________|
NAK Received                                                             ACK Received                      (I2C_WORD_DATA_READ)           
Send Stop Condition                                             Read Register Data MSB (8 bits)            (I2C_WORD_DATA_READ)             - 0x18
Clear I2C Bus (I2C_STOP_CONDITION)                                    Send ACK Bit (1Bit)                  (I2C_SEND_ACK_NAK_DATA_READ)     - 0x16
                                                                               |
                                                                       Send Stop Condition                 (I2C_PROCESS_STOP_CONDITION)     - 0x05
                                                                         Clear I2C Bus                     (I2C_STOP_CONDITION)             - 0x06

                                   Transmit Path (WORD)
                     Start Condition + 8 bit Address plus Write                                            (I2C_START_CONDITION)             - 0x00
                                 Read ACk bit (1 Bit)                                                      (I2C_READ_ADDRESS_ACK_NAK)        - 0x01
    _____________________________________|______________________________________
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_ADDRESS)       - 0x02
Send Stop Condition                                             Transmit Register Address (8 bits)         (I2C_CHECK_ACK_NAK_ADDRESS)
Clear I2C Bus (I2C_STOP_CONDITION)                                    Read ACK Bit (1Bit)                  (I2C_READ_ACK_NAK_REGISTER)       - 0x03
    ___________________________________________________________________________|
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_REGISTER)      - 0x04       
Send Stop Condition                                             Transmit Register Data LSB (8 bits)        (I2C_CHECK_ACK_NAK_REGISTER)
Clear I2C Bus (I2C_STOP_CONDITION)                                    Read ACK Bit (1Bit)                  (I2C_READ_ACK_NAK_DATA_WRITE)     - 0x20  
    ___________________________________________________________________________|
NAK Received                                                             ACK Received                      (I2C_CHECK_ACK_NAK_DATA_WRITE)    - 0x22      
Send Stop Condition                                             Transmit Register Data MSB (8 bits)        (I2C_CHECK_ACK_NAK_DATA_WRITE)
Clear I2C Bus (I2C_STOP_CONDITION)                                    Read ACK Bit (1Bit)                  (I2C_READ_ACK_NAK_DATA_LSB_WRITE) - 0x24
                                                                               |
                                                                       Send Stop Condition                 (I2C_PROCESS_STOP_CONDITION)      - 0x05
                                                                         Clear I2C Bus                     (I2C_STOP_CONDITION)              - 0x06
*/

#pragma vector = USI_VECTOR
__interrupt void USI_TXRX (void)
{
  static unsigned char LSB_Data;
  static unsigned char I2C_Pointer;
  I2C_State_Table [I2C_Pointer] = I2C_State;
  I2C_Pointer++;

  switch(I2C_State)
    {

/**************************************************************************************************************************************************
*    This Interrupt state is triggered by  Device Address and R/W bit are written to USISRL                                                       *
**************************************************************************************************************************************************/
    case (I2C_START_CONDITION):                                                 // Generate Start Condition & send address to slave
         // Track the state and signal start of I2C transaction
              clear_I2C_State_Machine();                                        // Clear the State Machine
         // Generate Start Condition
              USISRL = 0x00;                                                    // Set shift register LSB low AND...
              USICTL0 |= USIGE+USIOE;                                           // Set SDA = Output and set the Latch to Transparent
         // Send Slave Address + Read/Write bit
              USICTL0 &= ~USIGE;                                                // Set the Latch to Gated by Clock
              USISRL = SLV_Addr + I2C_WRITE;                                    // Load the shift register to the transmit address, R/W = 0
              USICNT |= 0x08;                                                   // Clock 8 bits by setting Bit counter
         // Triggers next interrupt after 8 clock cycles
              I2C_State = I2C_READ_ADDRESS_ACK_NAK;                             // Go to next state: receive address (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 Clocks which Transmits Slave Address + R/W Bit Transmitted, Ack/Nak bit is set up to be  received     *
**************************************************************************************************************************************************/
      case (I2C_READ_ADDRESS_ACK_NAK):                                          // Receive Address Ack/Nack bit
           // Read (N)ACK Bit
              read_ACK_NAK_bit ();           
           // Triggers next interrupt after 1 clock cycle
              I2C_State = I2C_CHECK_ACK_NAK_ADDRESS;                            // Go to next state: check (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 Clock which reads the Ack/Nak bit, If NAK, Stop Condition is sent, if ACK, MST_Register is sent       *
**************************************************************************************************************************************************/
      case (I2C_CHECK_ACK_NAK_ADDRESS):                                         // Process Address Ack/Nack & handle data TX
              USICTL0 |= USIOE;                                                 // Set SDA = output
           // Check Shift Register for a NACK
              if (USISRL & 0x01)                                                // If Nack received...
              {                                                                 
           // Send Stop Condition ...
                initiate_I2C_stop ();                                           // Triggers next interrupt after 1 clock cycle
           // Triggers next interrupt after 1 clock cycle
                I2C_State = I2C_STOP_CONDITION;                                 // Go to next state: generate Stop
              }
           // otherwise I2C is ACK'd
              else
              {                                                                 // Ack received, TX data to slave...
           // Send Device Register Address
                send_I2C_byte (REG_Addr);
           // Triggers next interrupt after 8 clock cycle
                I2C_State = I2C_READ_ACK_NAK_REGISTER;                          // Go to next state: receive data (N)Ack
              }
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 Clocks, which transmits the Register Address, Ack/Nak bit is set up to be  received                   *
**************************************************************************************************************************************************/
      case (I2C_READ_ACK_NAK_REGISTER):                                         // Receive Address Ack/Nack bit
           // Read (N)ACK Bit
              read_ACK_NAK_bit ();           
           // Triggers next interrupt after 1 clock cycle
              I2C_State = I2C_CHECK_ACK_NAK_REGISTER;                           // Go to next state: check (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 Clock, which reads the ACK/NAK Bit.   If NAK, Stop Condition is sent, if ACK, MST_DATA is transmitted *
*     or Restart Depending on the mode setting (READ or WRITE)                                                                                    *
**************************************************************************************************************************************************/
      case (I2C_CHECK_ACK_NAK_REGISTER):                                        // Process Address Ack/Nack & handle data TX
              USICTL0 |= USIOE;                                                 // Set SDA = output
           // Check Shift Register for a NACK
              if (USISRL & 0x01)                                                // If Nack received...
              {                                                               
           // Send Stop Condition ...
                initiate_I2C_stop ();                                           // Triggers next interrupt after 1 clock cycle
           // Triggers next interrupt after 1 clock cycle
                I2C_State = I2C_STOP_CONDITION;                                 // Go to next state: generate Stop
              }
           // otherwise I2C is ACK'd
              else
               {
           // Are we in Read Mode and need a Repeat Start?
                if (Read_Write_Mode == I2C_READ)
                {
                  USISRL = 0xFF;                                                // Set shift register LSB to 0xFF making SDA High
                  USICTL0 |= USIGE;                                             // Set the Latch to gated
                  USICNT |=  0x01;                                              // Bit counter = 1, making SCL high
           // Triggers next interrupt after 1 clock cycle
                  I2C_State = I2C_RESTART_CONDITION;                            // Go to next state: RESTART started
                }
           // otherwise we are in Write Mode
                else
                {                                                               // Ack received, TX data to slave...
           // Are we transmitting a Word?
                  if (Byte_Word_Mode == I2C_WORD)                               // If we are sending a word we need to send the MSB 1st
                  {
           // If we are in Word mode, we need to store the LSB and Shift the MSB 
                    LSB_Data = MST_Data;                                        // Store Master data LSB
                    MST_Data >>= 8;                                             // Shift Master data
           // Triggers next interrupt after 8 clock cycle
                    I2C_State = I2C_READ_ACK_NAK_DATA_WRITE;                    // Go to next state: receive data (N)Ack
                  }
           // otherwise we are in Byte Mode
                  else
                  {
           // Triggers next interrupt after 8 clock cycle
                    I2C_State = I2C_READ_ACK_NAK_DATA_LSB_WRITE;               // Go to next state: receive LSB data (N)Ack                    
                  }
           // Send 1st Data Byte
                  send_I2C_byte (MST_Data);
                }
              }
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 clock, which initiates the Re-Start                                                                   *
**************************************************************************************************************************************************/
      case (I2C_RESTART_CONDITION):
         // Generate Start Condition
              USISRL = 0x00;                                                    // Set shift register LSB low AND...
              USICTL0 |= USIGE+USIOE;                                           // Set SDA = Output and set the Latch to Transparent
         // Send Slave Address + Read/Write bit
              USICTL0 &= ~USIGE;                                                // Set the Latch to Gated by Clock
              send_I2C_byte ((SLV_Addr + I2C_READ));
         // Triggers next interrupt after 8 clock cycle
              I2C_State = I2C_READ_ACK_NAK_RESTART;                             // Go to next state: receive address (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 clocks, which transmits the Slave Address and the Read bit, Ack/Nak bit is set up to be received      *
**************************************************************************************************************************************************/
      case (I2C_READ_ACK_NAK_RESTART):                                          // Receive Address Ack/Nack bit
           // Read (N)ACK Bit
              read_ACK_NAK_bit ();           
           // Triggers next interrupt after 1 clock cycle
              I2C_State = I2C_CHECK_ACK_NAK_RESTART;                            // Go to next state: check (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 clocks, which transits the Data Word, Ack/NAK bit is set up to be received                            *
**************************************************************************************************************************************************/
      case (I2C_READ_ACK_NAK_DATA_WRITE):                                       // Receive Data Ack/Nack bit
           // Read (N)ACK Bit
              read_ACK_NAK_bit ();           
           // Triggers next interrupt after 1 clock cycle
              I2C_State = I2C_CHECK_ACK_NAK_DATA_WRITE;                         // Go to next state: check (N)Ack
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 clock, which receives the Ack/Nak bit, If NAK or Byte Write, Stop Condition is sent,                  *
*     else other Data Byte is Written                                                                                                             *
**************************************************************************************************************************************************/
      case (I2C_CHECK_ACK_NAK_DATA_WRITE):                                      // Process Data Ack/Nack
              USICTL0 |= USIOE;                                                 // Set SDA = output
           // Check Shift Register for a NACK
              if (USISRL & 0x01)                                                // If Nack received...
              {                                                               
           // Send Stop Condition ...
                initiate_I2C_stop ();                                           // Triggers next interrupt after 1 clock cycle
           // Triggers next interrupt after 1 clock cycle
                I2C_State = I2C_STOP_CONDITION;                                 // Go to next state: generate Stop
              }
           // otherwise I2C is ACK'd
              else                                                              // Ack received
              {
           // Load the LSB data into the Shift Register
                send_I2C_byte (LSB_Data);
           // Triggers next interrupt after 8 clock cycle
                I2C_State = I2C_READ_ACK_NAK_DATA_LSB_WRITE;                    // Go to next state: receive data (N)Ack
              }
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 clocks, which transits the Data Word, Ack/NAK bit is set up to be received                            *
**************************************************************************************************************************************************/
      case (I2C_READ_ACK_NAK_DATA_LSB_WRITE):                                   // Receive Data Ack/Nack bit
           // Read (N)ACK Bit
              read_ACK_NAK_bit ();           
           // Triggers next interrupt after 1 clock cycle
              I2C_State = I2C_PROCESS_STOP_CONDITION;                           // Go to next state: check (N)Ack
              break;              
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 clock, which receives the Ack/Nak, If NAK, Stop Condition is sent, Data Byte is Read                  *
**************************************************************************************************************************************************/
      case (I2C_CHECK_ACK_NAK_RESTART):                                         // Process Data Ack/Nack
           // Check Shift Register for a NACK
              if (USISRL & 0x01)                                                // If Nack received...
              {                                                               
           // Send Stop Condition ...
                USICTL0 |= USIOE;                                               // Set SDA = output
                initiate_I2C_stop ();                                           // Triggers next interrupt after 1 clock cycle
           // Triggers next interrupt after 1 clock cycle
                I2C_State = I2C_STOP_CONDITION;                                 // Go to next state: generate Stop
              }
           // otherwise I2C is ACK'd
              else                                                              // Ack received
              {
                read_I2C_byte ();
           // Triggers next interrupt after 8 clock cycles
                I2C_State = I2C_SEND_ACK_NAK_DATA_READ;                           // Go to next state: receive data (N)Ack
              }
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 8 clocks, receiving the read data, an ACK is sent, if word needed another read is done,                 *
*     if byte, stop initiated                                                                                                                     *
**************************************************************************************************************************************************/
     case (I2C_SEND_ACK_NAK_DATA_READ):                                         // Send Data Ack/Nack bit             
           // Are we in Word Mode?
              if (Byte_Word_Mode == I2C_WORD)
              {                                                                 // If this is not the last byte
                MST_Data = USISRL;                                              // Load data byte
                Byte_Word_Mode = I2C_BYTE;                                      // Clear to Byte read to stop the multi pass through the state machine
           // Triggers next interrupt after 1 clock cycle
                I2C_State = I2C_WORD_DATA_READ;                                // Go to next state: data/rcv again
              }
           // otherwise we are in Byte mode
              else                                                              // last byte: send NACK
              {
           // Shift MSB up and read byte from shift register in the LSB
                MST_Data <<= 8;                                                 // Shift Master data
                MST_Data |= USISRL;                                             // Load data byte
           // Triggers next interrupt after 1 clock cycles
                I2C_State = I2C_PROCESS_STOP_CONDITION;                         // stop condition
              }
           // Send ACK to Slave
              transmit_ACK_NAK_bit (I2C_ACK);              
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 clock transmitting an Ack to the Slave and reads the next 8 bits                                      *
**************************************************************************************************************************************************/
    case (I2C_WORD_DATA_READ):
           // Read in Next Data Byte
              read_I2C_byte ();     
           // Triggers next interrupt after 1 clock cycles
              I2C_State = I2C_SEND_ACK_NAK_DATA_READ;                           // Go to next state: receive data (N)Ack
              break;      
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by 1 clock, the Stop condition is sent                                                                     *
**************************************************************************************************************************************************/
     case I2C_PROCESS_STOP_CONDITION:
           // Send stop...
              initiate_I2C_stop ();
           // Triggers next interrupt after 1 clock cycles
              I2C_State = I2C_STOP_CONDITION;                                   // Go to next state: generate Stop
              break;
/**************************************************************************************************************************************************
*    This Interrupt state is triggered by Stop bit received, State machine is reset for next transfer                                             *
**************************************************************************************************************************************************/
      case (I2C_STOP_CONDITION):                                                // Generate Stop Condition
           // Transition SDA high to complete stop
              USISRL = 0xFF;                                                    // Shift register LSB = 1 to signal stop
              USICTL0 |= USIGE;                                                 // Transparent latch enabled
           // Release I2C
              USICTL0 &= ~(USIGE+USIOE);                                        // Latch/SDA output disabled, releasing the SDA
              I2C_State = I2C_START_CONDITION;                                  // Reset state machine for next transmission
              LPM0_EXIT;                                                        // Exit active for next transfer
              I2C_Pointer = 1;                                                  // Reset Table Pointer
              break;
    }

  USICTL1 &= ~USIIFG;                                                           // Clear pending flag
}
